@@ -1,7 +1,7 @@
 
 import sys, os, re, subprocess
 
-from nesoni import grace, config, legion, io, workspace
+from nesoni import grace, config, legion, io, workspace, selection
 
 def R_literal(item):
     if item is None:
@@ -251,15 +251,18 @@ to expression level.
 
 A plot ...-qq.png is produced showing how well the chi-square 
 distribution fits the residual variance.
-
-"voomed" columns in the output are log2 reads per million,
-normalized using EdgeR's Trimmed Mean normalization.  
 """,
-'log' : """
+'glog' : """
 This is a simpler alternative to voom. Counts are converted to 
-log2 Reads Per Million using the following formula:
+log2 Reads Per Million using a variance stabilised transformation.
 
-   log2( (count/library_size + log_moderation/mean_library_size)*1e6 )
+Let the generalised logarithm with moderation m be
+
+   glog(x,m) = log2((x+sqrt(x*x+4*m*m))/2)
+
+then the transformed values will be
+
+   glog( count/library_size*1e6, log_moderation/mean_library_size*1e6 )
 
 where log_moderation is a parameter.
 
@@ -267,9 +270,33 @@ The log2 RPM values are then tested using limma.
 
 References:
 
-Smyth, G. K. (2004). Linear models and empirical Bayes methods for assessing 
-differential expression in microarray experiments. Statistical Applications 
-in Genetics and Molecular Biology 3, No. 1, Article 3.
+B.P. Durbin, J.S. Hardin, D.M. Hawkins and D.M. Rocke (2002)
+A variance-stabilizing transformation for gene-expression microarray data.
+Bioinformatics (2002) 18 (suppl 1): S105-S110. 
+
+Smyth, G. K. (2004). 
+Linear models and empirical Bayes methods for assessing 
+differential expression in microarray experiments. 
+Statistical Applications in Genetics and Molecular Biology 3, No. 1, Article 3.
+""",
+'nullglog' : """
+Uses BioConductor package limma, after tranformation as in the "glog" method, 
+to fit the null model to the data. A scaled chi-square distribution is then 
+fitted to the residual variance, using maximum likelihood. Genes where the 
+residual is significantly larger than would be expected from the fitted 
+chi-square distribution are reported as significant.
+
+Use this when your biologist (bless them) has not done any 
+biological replicates.
+
+This mode is unable to distinguish differentially expressed genes from 
+genes with high variability between samples.
+
+This mode gives quite conservative p-values. Also note that this is
+not a method endorsed by WEHI, it's just something I made up.
+
+A plot ...-qq.png is produced showing how well the chi-square 
+distribution fits the residual variance.
 """,
 'poisson' : """     
 Uses BioConductor package EdgeR to look for differential expression 
@@ -366,9 +393,11 @@ do.heatmap <- function(heatmap.features, heatmap.data, heatmap.labels, heatmap.v
                 title=colnames(heatmap.values)[i]
             )
     
+    heatmap.labels.ord <- lapply(heatmap.labels,function(item) item[ord])
+    
     multiplot(
         cols,
-        heatmap.labels
+        heatmap.labels.ord
     )           
     dev.off()
 }           
@@ -413,13 +442,20 @@ if (nrow(dgelist$counts) == 0) {
     if (MODE == 'trend') {
         cat('estimateGLMTrendedDisp\n')
         d <- estimateGLMTrendedDisp(d, design=design)
-        dispersion <- d$trended.dispersion
+        #dispersion <- d$trended.dispersion
+        cat('estimateGLMTagwiseDisp\n')
+        d <- estimateGLMTagwiseDisp(d, design=design)
+        dispersion <- d$tagwise.dispersion
         summary(dispersion)
     } else if (MODE == 'common') {
         cat('estimateGLMCommonDisp\n')
         d <- estimateGLMCommonDisp(d, design=design)
-        dispersion <- d$common.dispersion
-        print(dispersion)
+        #dispersion <- d$common.dispersion
+        print(d$common.dispersion)
+        cat('estimateGLMTagwiseDisp\n')
+        d <- estimateGLMTagwiseDisp(d, design=design)
+        dispersion <- d$tagwise.dispersion
+        summary(dispersion)
     } else {
         cat('Poisson dispersion\n')
         dispersion <- 1e-6    #See edgeR Poisson example
@@ -543,7 +579,8 @@ if (nrow(dgelist$counts) == 0) {
     
     
     heatmap.features <- rev(as.character(result$Feature[significant]))
-    heatmap.data <- voom(dgelist)$E[heatmap.features,,drop=FALSE]
+    heatmap.data <- glog2.rpm.counts(dgelist, GLOG_MODERATION)$E[heatmap.features,,drop=FALSE]
+       #voom(dgelist)$E[heatmap.features,,drop=FALSE]
     heatmap.labels <- list(heatmap.features)
     heatmap.values <- result[heatmap.features,colnames(design)[1:N_TO_TEST],drop=FALSE]    
     for(i in basic.seq(ncol(heatmap.values))) {
@@ -607,16 +644,18 @@ LIMMA = COMMON + r"""
 
 library(limma)
 
-if (MODE == 'log') {
-    y <- log.rpm.counts(dgelist, LOG_MODERATION)
+null.design <- design[, (N_TO_TEST+1):ncol(design), drop=FALSE]
+
+if (MODE == 'glog' || MODE == 'nullglog') {
+    y <- glog2.rpm.counts(dgelist, GLOG_MODERATION)
 
     if (QUANTILE_NORM) {
         y$E <- normalizeBetweenArrays(y$E, method = 'quantile')
     }
 
 } else {
-    if (MODE == 'nullvoom') {
-        voom.design <- design[, (N_TO_TEST+1):ncol(design), drop=FALSE]
+    if (MODE == 'nullvoom' || MODE == 'nullglog') {
+        voom.design <- null.design
     } else {
         voom.design <- design
     }
@@ -640,13 +679,13 @@ if (USE_CONTRAST) {
     coef <- fit$coefficients[,1:N_TO_TEST, drop=FALSE]
 }
 
-if (MODE != 'nullvoom') {
+if (MODE != 'nullvoom' && MODE != 'nullglog') {
     fit <- eBayes(fit)
     
     if (USE_CONTRAST) {
-        top <- topTable(fit, coef=1, sort.by='none', number=Inf)
+        top <- topTable(fit, coef=1, sort.by='none', confint=TRUE, number=Inf)
     } else {    
-        top <- topTable(fit, coef=1:N_TO_TEST, sort.by='none', number=Inf)
+        top <- topTable(fit, coef=1:N_TO_TEST, sort.by='none', confint=N_TO_TEST==1, number=Inf)
     }
      
     ordering <- order(top$P.Value)
@@ -655,9 +694,11 @@ if (MODE != 'nullvoom') {
 } else {
     stopifnot(!USE_CONTRAST) #Not supported.
 
-    nullfit <- lmFit(y, voom.design)
-    df <- ncol(dgelist$counts) - ncol(voom.design)
-    s2 <- nullfit$sigma ^ 2
+    fit <- lmFit(y, null.design)
+    fit <- eBayes(fit)
+    
+    df <- ncol(dgelist$counts) + fit$df.prior - ncol(null.design)
+    s2 <- fit$s2.post  #fit$sigma ^ 2
     
     #Trimmed chisq (untrimmed turns out to work just as well)
     #cut <- quantile(s2, 0.99)[[1]]
@@ -680,12 +721,16 @@ if (MODE != 'nullvoom') {
     #)    
     
     #Maximum likelihood scale of chi-square (special case of gamma distribution)
-    scale <- df*length(s2)/sum(s2)
+    #scale <- df*length(s2)/sum(s2)
+    
+    # fit$s2.prior allows that a proportion (1%) of genes are differentially expressed
+    # Similar idea to trimmed chisq, above.
+    scale <- df / fit$s2.prior
 
     png(sprintf('%s-qq.png', OUTPUT_PLOT), width=800, height=800 )
     qqplot(qchisq( ((1:length(s2))-0.5)/length(s2) , df)/scale, s2,
         xlab='Fitted chi-square quantiles',
-        ylab='Sample quantiles',
+        ylab='sigma^2',
         main='QQ plot of sigma^2 unexplained by null model vs chi-squared distribution' 
     ) 
     abline(0,1)
@@ -703,6 +748,10 @@ for(i in basic.seq(ncol(coef))) {
     result[,colnames(coef)[i]] <- coef[,i]
 }
 
+if ((MODE == 'glog' || MODE == 'voom') && ncol(coef) == 1) {
+    result[,'+/- at 95% confidence'] <- (top$CI.975-top$CI.025)*0.5
+}
+
 result$p <- p
 
 result$FDR <- p.adjust(p)
@@ -739,33 +788,43 @@ sink(OUTPUT_FILENAME)
 write.csv(results_to_output[,!blank,drop=FALSE], na='', row.names=FALSE)
 sink()
 
-if (ncol(coef) == 1) # <-- Plots don't seem useful for ANOVA
-    for(i in 1:ncol(coef)) {
-        colname <- colnames(coef)[i]    
-        
-        #Strip "log2 "
-        name <- substr(colname, 6,nchar(colname))
-        
-        if (ncol(coef) == 1) {
-            pngname = sprintf('%s.png', OUTPUT_PLOT)
-        } else {
-            pngname = sprintf('%s-%s.png', OUTPUT_PLOT, name) 
-        }
+if (ncol(coef) == 1) { # <-- Plots don't seem useful for ANOVA
+    i <- 1    
+    colname <- colnames(coef)[i]    
     
-        png(pngname, width=800, height=800 )
-        plot(result[,"average expression (log2 reads-per-million)"], 
-             result[,colname],
-             xlab="log2 concentration",
-             ylab="log2 fold change",
-             main=name,
-             pch=19, cex=0.25)
-             
-        points(result[,"average expression (log2 reads-per-million)"][significant],
-               result[,colname][significant],
-               pch=19, cex=1.0,
-               col="red")
-        dev.off()
-    }
+    #Strip "log2 "
+    name <- substr(colname, 6,nchar(colname))
+    
+    pngname = sprintf('%s.png', OUTPUT_PLOT)
+
+    png(pngname, width=800, height=800 )
+
+    plot(result[,"average expression (log2 reads-per-million)"], 
+         result[,colname],
+         xlab="log2 concentration",
+         ylab="log2 fold change",
+         main=name,
+         pch=19, cex=0.25)
+    
+    #if (MODE == 'glog' || MODE == 'voom') {
+    #    title(xlab='Error bars indicate 95% confidence interval',adj=1)
+    #    
+    #    use <- abs(result[,colname]) >= sort(abs(result[,colname]))[floor(1+nrow(result) * 0.99)]
+    #    arrows(result[,"average expression (log2 reads-per-million)"][use],
+    #           y0=top[ordering,'CI.025'][use],
+    #           y1=top[ordering,'CI.975'][use],
+    #           angle=90,code=3,length=0.03,col=c('#8888ff'))  
+    #    points(result[,"average expression (log2 reads-per-million)"], 
+    #         result[,colname],
+    #         pch=19, cex=0.25)
+    #}
+        
+    points(result[,"average expression (log2 reads-per-million)"][significant],
+           result[,colname][significant],
+           pch=19, cex=1.0,
+           col="red")
+    dev.off()
+}
 
 
 heatmap.features <- rev(as.character(result$Feature[significant]))
@@ -785,17 +844,17 @@ for(i in 1:ncol(dgelist$counts)) {
 }
 cat('\n\n')
 
-if (MODE == 'log') {
-    cat('Using log_moderation =',LOG_MODERATION,'\n\n')
+if (MODE == 'glog' || MODE == 'nullglog') {
+    cat('Using glog_moderation =',GLOG_MODERATION,'\n\n')
 }
 
 cat(dgelist$original.number.of.genes, 'features\n')
 cat('Discarded', dgelist$original.number.of.genes-nrow(dgelist$counts), 'features with total count less than', MIN_COUNT, '\n')
 cat('Kept', nrow(dgelist$counts), 'features\n')
 
-if (MODE == 'voom' || MODE == 'log') {
-    cat(fit$df.prior, 'prior df (the prior is like this many extra samples)\n')
-}
+#if (MODE == 'voom' || MODE == 'glog') {
+cat(fit$df.prior, 'prior df (the prior is like this many extra samples)\n')
+#}
 
 cat('With a False Discovery Rate of', FDR_CUTOFF, 'there are\n')
 cat(sum(significant), 'significantly differentially expressed tags\n\n')
@@ -831,29 +890,69 @@ def term_name(term):
 @config.help("""\
 Find significant differential expression from the output of "count:".
 
-Terms are a regular expression on the sample names, or several 
-terms joined by ^, indicating an interaction term (ie the XOR of the 
-component terms). "with:" terms are included in the linear model, but 
-not the significance test. The model will also include a constant term.
-
-Examples: A A^B
-
-A term can be given a more intuitive name in the output by appending "=name", eg
+Terms are selection expressions on the sample names and any tags they were \
+given with "tag:". See the main nesoni help page for selection expression syntax. \
+"with:" terms are included in the linear model, but not the significance test. \
+The model will also include a constant term.
 
+A term can be given a more intuitive name in the output by appending "=name". \
 Examples: A=fold-change A^B=interaction-between-A-and-B
+
+
+Example 1: Say we have two conditions, "control" and "experimental". Here's are some possible tests:
+
+experimental
+- What is the fold change from control to experimental? \
+Hypothesis 0 is that all samples have the same log expression level. \
+Hypothesis 1 is that the control samples have a baseline log expression level, \
+and the experimental samples have that baseline plus a further amount.
+
+control experimental contrast: -1 1 --constant-term no
+- This is a different way to perform the same test.
+
+
+Example 2: Say we have two strains, "strain1" and "strain2" and two time points "time1" and "time2". \
+Here are some possible tests:
+
+time2
+- What is the fold change from time1 to time2? \
+But note that any difference between strains will be seen as within group variation!
+
+time2 --select strain1
+- What is the fold change from time1 to time2, looking at strain1 samples only?
+
+time2 with: strain2
+- What is there a change from time1 to time2? \
+Any difference between strains is accounted for.
+
+strain2^time2 with: strain2 time2
+- Is there an interaction effect between strain and time?
+
+
+Example 3: Say we have three time points, "time1", "time2" and "time3". Here are some possible tests:
+
+time3 --select time2/time3
+- Is there a change between time2 and time3?
+
+time2 time3
+- Is there any change over time? \
+This is an ANOVA style test that is potentially more sensitive than comparing each pair of time points individually.
+
+
 """ +
 ''.join(
         '\n\n'+
         '='*20 +
         '\n--mode %s \n' % mode +
         MODE_HELP[mode]        
-        for mode in ('voom', 'log', 'nullvoom', 'poisson', 'common', 'trend')
+        for mode in ('voom', 'glog', 'nullvoom', 'nullglog', 'poisson', 'common', 'trend')
 ))
 @config.String_flag('mode', 'Analysis mode. See below.')
-@config.String_flag('log_moderation', 'For "--mode log" only.'
-    ' Expression levels are calculated as'
-    ' log2( (count/library_size + log_moderation/mean_library_size)*1e6 ).'
+@config.Float_flag('glog_moderation', 
+    'For "--mode glog" only.'
+    ' Amount of moderation used in log transformation.'
     )
+@config.String_flag('select', 'Selection expression. Which samples to use.')
 @config.Int_flag('min_count', 'Discard features with less than this total count.')
 @config.Float_flag('fdr', 'False Discovery Rate cutoff for statistics and plots.')
 @config.Bool_flag('output_all', 'List all genes in output, not just significant ones.')
@@ -864,12 +963,13 @@ Examples: A=fold-change A^B=interaction-between-A-and-B
 @config.Main_section('test', 'Terms to test.')
 @config.Float_section('contrast', 'Instead of doing an ANOVA on multiple terms, perform a contrast with the given weights.')
 @config.Section('with_', 'Also include these terms in the model.')
-@config.Section('use', 'Only use samples matching these regexes.')
+#@config.Section('use', 'Only use samples matching these regexes.')
 class Test_counts(config.Action_with_prefix):
+    select = 'all'
     min_count = 10
     constant_term = True
     mode = 'voom'
-    log_moderation = '5.0'
+    glog_moderation = 5.0
     quantile_norm = False
     fdr = 0.01
     output_all = True
@@ -884,11 +984,13 @@ class Test_counts(config.Action_with_prefix):
 
     def run(self):
         test_counts_run(    
-            min_count=self.min_count, constant_term=self.constant_term, mode=self.mode, log_moderation=self.log_moderation,
+            min_count=self.min_count, constant_term=self.constant_term, mode=self.mode, glog_moderation=self.glog_moderation,
             quantile_norm=self.quantile_norm, fdr=self.fdr, output_all=self.output_all, 
             only_tell=self.tell,
             output_prefix=self.prefix, filename=self.counts_file, 
-            test_terms=self.test, with_terms=self.with_, contrast_weights=self.contrast, use_terms=self.use,
+            test_terms=self.test, with_terms=self.with_, contrast_weights=self.contrast, 
+            #use_terms=self.use,
+            select=self.select,
             norm_file=self.norm_file,
         )
 
@@ -947,16 +1049,18 @@ class Test_counts(config.Action_with_prefix):
 #    )
 
 def test_counts_run(
-    min_count, constant_term, mode, log_moderation, quantile_norm, fdr, output_all, only_tell,
-    output_prefix, filename, test_terms, with_terms, contrast_weights, use_terms,
+    min_count, constant_term, mode, glog_moderation, quantile_norm, fdr, output_all, only_tell,
+    output_prefix, filename, test_terms, with_terms, contrast_weights, 
+    #use_terms,
+    select,
     norm_file
 ):
     log = grace.Log()
 
-    assert mode in ('voom', 'log', 'nullvoom', 'poisson', 'common', 'trend')
+    assert mode in ('voom', 'glog', 'nullvoom', 'nullglog', 'poisson', 'common', 'trend')
     
-    if not use_terms:
-        use_terms = [''] #Default to all
+    #if not use_terms:
+    #    use_terms = [''] #Default to all
     
     assert test_terms, 'No terms to test'
     
@@ -964,7 +1068,7 @@ def test_counts_run(
     assert not use_contrast or len(contrast_weights) == len(test_terms), 'Need one contrast weight per term'
     
     all_terms = test_terms + with_terms 
-    if constant_term: all_terms += [ '' ]
+    if constant_term: all_terms += [ 'all' ]
     n_to_test = len(test_terms)
     
     if use_contrast:
@@ -978,30 +1082,45 @@ def test_counts_run(
     #n_all_samples = parts.index('RPKM '+parts[1])-1    
     #all_samples = parts[1:n_all_samples+1]
     
-    data = io.read_grouped_table(filename,{'Count':str,'Annotation':str})
-    assert len(data['Count']), 'Count file is empty'
+    #data = io.read_grouped_table(filename,{'Count':str,'Annotation':str})
+    #assert len(data['Count']), 'Count file is empty'    
+    #all_samples = data['Count'].values()[0].keys()
+    
+    reader = io.Table_reader(filename, 'Count')
+    reader.close()
+    all_samples = [ item for i, item in enumerate(reader.headings) if reader.groups[i] == 'Count' ]
+    tags = { }
+    for item in all_samples:
+        tags[item] = [ item ]
+    for line in reader.comments:
+        if line.startswith('#sampleTags='):
+            parts = line[len('#sampleTags='):].split(',')
+            tags[parts[0]] = parts
     
-    all_samples = data['Count'].values()[0].keys()
     n_all_samples = len(all_samples)
-    
-    
-    keep = [ any( re.search(use_expr, item) for use_expr in use_terms ) for item in all_samples ]
+        
+    #keep = [ any( re.search(use_expr, item) for use_expr in use_terms ) for item in all_samples ]
+    keep = [ selection.matches(select, tags[item]) for item in all_samples ]
     samples = [ all_samples[i] for i in xrange(n_all_samples) if keep[i] ]
     n_samples = len(samples)
     
+    assert n_samples, 'No samples selected'
+    assert n_samples > 1, 'Only one sample selected'
+    
     model = [ ]
-    for term in all_terms:
-        column = [ False ] * n_samples
-        for expression in term_specification(term).split('^'):
-            for i in xrange(n_samples):
-                if re.search(expression, samples[i]):
-                    column[i] = not column[i]
-        model.append([ int(item) for item in column ])
+    for term in all_terms:        
+        spec = term_specification(term)
+        model.append([ 1 if selection.matches(spec, tags[item]) else 0 for item in samples ])
+        #column = [ False ] * n_samples
+        #for expression in term_specification(term).split('^'):
+        #    for i in xrange(n_samples):
+        #        if re.search(expression, samples[i]):
+        #            column[i] = not column[i]
+        #model.append([ int(item) for item in column ])
     model = zip(*model) #Transpose
     
     
-    
-    
+        
     log.log('Model matrix (terms being tested for significance shown in { }):\n')
     for i in xrange(n_samples):
         log.log('{ ')
@@ -1018,7 +1137,7 @@ def test_counts_run(
     
     
     assert len(all_terms) <= n_all_samples, 'Can\'t have more linear model terms than samples.'
-    if mode not in ('poisson','nullvoom'):
+    if mode not in ('poisson','nullvoom','nullglog'):
         assert len(all_terms) < n_all_samples, (
             'Can\'t have as many linear model terms as samples, within group variation can\'t be estimated. '
             'I wouldn\'t recommend using --mode poisson or --mode nullvoom, '
@@ -1057,7 +1176,7 @@ def test_counts_run(
     log.attach(open(log_filename,'wb'))
     log.close()    
     
-    if mode in ('voom', 'log', 'nullvoom'):
+    if mode in ('voom', 'glog', 'nullvoom', 'nullglog'):
         script = LIMMA
     else:
         script = EDGER
@@ -1076,7 +1195,7 @@ def test_counts_run(
         OUTPUT_ALL = output_all,
         MIN_COUNT = min_count,
         MODE = mode,
-        LOG_MODERATION = log_moderation,
+        GLOG_MODERATION = glog_moderation,
         QUANTILE_NORM = quantile_norm,
         KEEP = keep,
         OUTPUT_FILENAME = output_prefix + '.csv',
@@ -1362,7 +1481,7 @@ if (length(ORDER)) {
 }
 
 #elist <- voom(dgelist, normalize.method=if(QUANTILE) 'quantile' else 'none')
-elist <- log.rpm.counts(dgelist, LOG_MODERATION)
+elist <- glog2.rpm.counts(dgelist, GLOG_MODERATION)
 
 hmap.elist(PREFIX, elist, min.sd=MIN_SD, min.span=MIN_SPAN, min.svd=MIN_SVD, svd.rank=SVD_RANK, reorder.columns=REORDER_COLUMNS)
 
@@ -1380,9 +1499,9 @@ Hierachical clustering and ordering of rows is performed using the "seriation" p
 
 You will need to use at least one of --min-sd, --min-span, or --min-svd.
 """)
-@config.String_flag('log_moderation',
-    ' Log RPM expression levels are calculated as'
-    ' log2( (count/library_size + log_moderation/mean_library_size)*1e6 ).'
+@config.Float_flag('glog_moderation', 
+    'Amount of moderation used in log transformation.'
+    ' See "glog" mode in "test-counts:".'
     )
 @config.Int_flag('min_total', 'Expression level filter:\nExclude genes with less than this total number of reads')
 @config.Int_flag('min_max', 'Expression level filter:\nExclude genes with no sample having at least this many reads')
@@ -1397,7 +1516,7 @@ You will need to use at least one of --min-sd, --min-span, or --min-svd.
 @config.Positional('counts', 'File containing output from "nesoni count:"')
 @config.Section('order', 'Optionally, specify an order to show the columns in.')
 class Heatmap(config.Action_with_prefix):
-    log_moderation = '5.0'
+    glog_moderation = 5.0
     counts = None
     min_total = 0
     min_max = 0
@@ -1413,7 +1532,7 @@ class Heatmap(config.Action_with_prefix):
     def run(self):
         run_script(HEATMAP_SCRIPT,
             PREFIX=self.prefix,
-            LOG_MODERATION=self.log_moderation,
+            GLOG_MODERATION=self.glog_moderation,
             COUNTS=self.counts,
             MIN_TOTAL=self.min_total,
             MIN_MAX=self.min_max,
